using System;
using System.Collections;
using System.Collections.Generic;

namespace Prova1
{
    class WorkQueue : IEnumerable<Work>
    {
        private readonly Queue<Work> _works = new Queue<Work>();

        public event WorkEventHandler Enqueued;
        public event WorkEventHandler Dequeued;

        public int Count
        {
            get { return _works.Count; }
        }

        public void Enqueue(Work work)
        {
            if (work == null)
                throw new ArgumentNullException("work == null");
            _works.Enqueue(work);
            OnEnqueued(work);
        }

        public Work Dequeue()
        {
            Work work = _works.Dequeue();
            OnDequeued(work);
            return work;
        }

        protected void OnEnqueued(Work work)
        {
            if (Enqueued != null)
                Enqueued(this, work);
        }

        protected void OnDequeued(Work work)
        {
            if (Dequeued != null)
                Dequeued(this, work);
        }

        #region IEnumerable<Work> Members

        public IEnumerator<Work> GetEnumerator()
        {
            return _works.GetEnumerator();
        }

        #endregion

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return _works.GetEnumerator();
        }

        #endregion
    }
}
