﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Prova1
{
    class Worker
    {
        private Work _work;
        private readonly int _id;

        private static int _nextId = 1;

        public event WorkEventHandler Idle;
        public event WorkEventHandler WorkDone;
        public event WorkEventHandler WorkStarted;

        public Worker()
        {
            _id = _nextId++;
        }

        public int Id
        {
            get { return _id; }
        }

        public Work Work
        {
            get { return _work; }
        }

        public bool IsWorking
        {
            get { return (Work != null); }
        }

        public void Assign(Work work)
        {
            if (IsWorking)
                throw new ApplicationException("IsWorking");
            _work = work;
            OnWorkStarted();
        }

        public void Pulse()
        {
            if (IsWorking)
            {
                _work.Do();
                if (_work.Done)
                {
                    OnWorkDone();
                    _work = null;
                    OnIdle();
                }
            }
            else
            {
                OnIdle();
            }
        }

        protected void OnIdle()
        {
            if (Idle != null)
                Idle(this, Work);
        }

        protected void OnWorkDone()
        {
            if (WorkDone != null)
                WorkDone(this, Work);
        }

        protected void OnWorkStarted()
        {
            if (WorkStarted != null)
                WorkStarted(this, Work);
        }
    }
}
